require 'ext/struct'

module PayloadLookup

  def [](key)
    payload[key]
  end

end

Event = Struct.new(:topic, :type, :payload) do

  include StructToJSON
  include StructToIGLU

  def self.parse(event_hash)
    event_hash = JSON.parse(event_hash).deep_symbolize_keys
    new(
      event_hash[:topic],
      event_hash[:type],
      event_hash[:payload],
    )
  end

  include PayloadLookup

end

UnifiedEvent = Struct.new(:topic, :type, :payload, :sequence, :tsequence, :timestamp) do

  include StructToJSON

  def self.from(event, sequence = nil, tsequence = nil, timestamp = nil)
    raise ArgumentError.new('missing sequence')  unless sequence
    raise ArgumentError.new('missing tsequence') unless tsequence
    raise ArgumentError.new('missing timestamp') unless timestamp
    new(event.topic, event.type, event.payload, sequence, tsequence, timestamp)
  end

  def self.parse(event_hash)
    event_hash = JSON.parse(event_hash).deep_symbolize_keys
    new(
      event_hash[:topic],
      event_hash[:type],
      event_hash[:payload],
      event_hash[:sequence],
      event_hash[:tsequence],
      event_hash[:timestamp],
    )
  end

  include PayloadLookup

end

ConsumerEvent = Struct.new(:topic, :type, :payload, :sequence, :tsequence, :timestamp, :replay) do

  include StructToJSON

  def self.from(unified_event, replay = nil)
    new(unified_event.topic, unified_event.type, unified_event.payload, unified_event.sequence, unified_event.tsequence, unified_event.timestamp, replay)
  end

  def replay?
    !!self.replay
  end

  def self.parse(event_hash)
    event_hash = JSON.parse(event_hash).deep_symbolize_keys
    new(
      event_hash[:topic],
      event_hash[:type],
      event_hash[:payload],
      event_hash[:sequence],
      event_hash[:tsequence],
      event_hash[:timestamp],
      event_hash[:replay],
    )
  end

  include PayloadLookup

end
